<?

 /***************************************************************************************
 * Filename: modules/categories.php
 *
 * This file is part of the Photo album JAS plugin.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The Photo album JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/photoalbum/
 ***************************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["include"]."fs.php");

 class Plugin_Module_photoalbum_categories extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->addField("published", "boolean", "all", "", 0, false, true, "Published");

   $this->data->addField("directory", "localdirectory", "all", "recursive", 1, 65536, false, "Photo album directory");

   $this->extend("categories");

   $this->data->title="Photo albums";

   $this->data->addField("date", "date", "all", "", 0, false, false, "Date");

   $this->data->addField("picture", "nr", "all", "", 0, false, false, "Album picture");
   $this->data->addReference("picture", "photoalbum_pictures", "nr", "*:0");

   //tags
   $this->data->addField("tags", "text", "show;form", "", 0, 255, false, "Tags");
   $this->data->addReference("photoalbum_tagsmapping", "photoalbum_tags", "nr", "*:*", "nr");

   if(isset($this->data->field["photoalbum_tagsmapping"]))
    $this->data->field["photoalbum_tagsmapping"]["properties"].=";hidden";

   $this->data->group("Picture", "picture");

   //force categories defaults
   $this->data->setValue("moduleLock", true);
   $this->data->setValue("module", Array("photoalbum_categories", "photoalbum_pictures"));

   //hide categories fields
   $this->data->field["ordernr"]["properties"].=";hidden";
   $this->data->field["moduleLock"]["properties"].=";hidden";
   $this->data->field["module"]["properties"].=";hidden";

   $this->data->sortOrder("date-", "name");
  }

  function postInit()
  {
   global $_path;

   $this->data->setFileDir("directory", $_path["project"]["root"]."photoalbums");

   return true;
  }

  /*
   * Pre get
   */

  function viewPreGetErrorCheck(){return $this->convertTextToTags();}


  /*
   * Post get
   */

  function viewPostGet()
  {
   if(!$this->getAlbumPicture())
    return false;

   if(!$this->convertTagsToText())
    return false;

   return true;
  }

  function copyPostGet(){return $this->convertTagsToText();}
  function editPostGet(){return $this->copyPostGet();}
  function movePostGet(){return $this->copyPostGet();}
  function deletePostGet(){return $this->copyPostGet();}
  function ownerPostGet(){return $this->copyPostGet();}


  /*
   * Pre submit
   */

  function addPreSubmitErrorCheck()
  {
   if(!$this->getNameAndDate())
    return false;

   if(!$this->convertTextToTagsAndAdd())
    return false;

   return true;
  }
  function editPreSubmitErrorCheck(){return $this->addPreSubmitErrorCheck();}


  /*
   * Support functions
   */

  function getAlbumPicture()
  {
   if($this->data->check("use", "show", "picture")) //show picture
   {
    $d=Data::cache("photoalbum_pictures", "view", "all");
    $d->setShowFields("nr");
    $d->clearValues();

    for($c=0;$c<$this->data->vmax;$c++)
    {
     if(!$this->data->getValue("picture", $c)) //no picture set, get random
     {
      $d->setValue("catnr", $this->data->getValue("nr", $c));

      if($d->get(false, true)&&($max=$d->getValue("show_count"))) //get total number of pictures in album
      {
       $d->clearValues();
       $d->setValue("catnr", $this->data->getValue("nr", $c));
       $d->setValue("show_from", rand(0, $max-1)); //set random limit offset
       $d->setValue("show_total", 1); //only get 1 entry

       if($d->get()) //get nr
        $this->data->setValue("picture", $d->getValue("nr"), $c);
      }
     }
    }
   }

   return true;
  }


  function convertTagsToText()
  {
   if($this->data->check("use", "show", "tags"))
   {
    $d=&$this->data->reference["photoalbum_tagsmapping"]["data"]; //get photoalbum_tagsmapping

    //setup photoalbum_tags reference
    if(!isset($d->reference["photoalbum_tags"]["data"]))
     $d->initReferenceData("photoalbum_tags");

    $showFields=$d->reference["photoalbum_tags"]["data"]->check("use", "show"); //store show fields
    $d->reference["photoalbum_tags"]["data"]->setShowFields("tag");

    for($c=0;$c<$this->data->vmax;$c++)
    {
     $d->setAction("view");
     $d->setExtra("all");
     $d->clearValues();
     $d->setValue($this->data->module, $this->data->getValue("nr", $c));

     $tags=Array();
     if($d->get(1))
     {
      while($d->next())
       $tags[]=$d->v["photoalbum_tags"]["tag"];
     }

     $this->data->setValue("tags", implode(", ", $tags), $c);
    }

    $d->reference["photoalbum_tags"]["data"]->setShowFields($showFields); //restore show fields
    unset($d->reference["photoalbum_tags"]["data"]); //delete photoalbum_tags data object
   }

   return true;
  }

  function convertTextToTags()
  {
   $tagsmapping=0;
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("photoalbum_tags", "view", "all");
      $d->setShowFields("nr");
     }

     $x=0;
     $d->clearValues();
     foreach($text as $tag) //find correspondig tag nr
     {
      $d->setValue("tag", $tag, $x);
      $d->setOperant("tag", "OR", $x);
      $x++;
     }

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }
     else
     {
      if($d->vmax==0) //tag(s) not found
      {
       $this->data->setValue("photoalbum_tagsmapping", 0, $tagsmapping); //ensure no results are return (unless other tags are found)
       $this->data->setOperant("photoalbum_tagsmapping", "OR", $tagsmapping);
       $tagsmapping++;
      }
      else
      {
       while($d->next())
       {
        $this->data->setValue("photoalbum_tagsmapping", $d->v["nr"], $tagsmapping);
        $this->data->setOperant("photoalbum_tagsmapping", "OR", $tagsmapping);
        $tagsmapping++;
       }
      }
     }
    }
   }

   return true;
  }

  function convertTextToTagsAndAdd()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("photoalbum_tags", "view", "all");
      $d->setShowFields("nr");

      //get tags category
      if(!($tagsCatNR=photoalbum_categories::getCategory("tags")))
      {
       $this->data->e->add("Unable to add tags, no tags category found", "error");
       return false;
      }
     }

     foreach($text as $tag) //find correspondig tag nr
     {
      $d->clearValues();
      $d->setValue("tag", $tag);

      if($d->get()&&$d->vmax) //tag exists
       $tags[]=$d->getValue("nr"); //store tag+id
      else
      {
       //add tag to database
       $d->setAction("add");
       $d->setExtra("item");

       $d->clearValues();
       $d->setValue("tag", $tag);
       $d->setValue("catnr", $tagsCatNR);

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }

       $tags[]=$d->getValue("nr"); //store tag+id

       //reset data object
       $d->setAction("view");
       $d->setExtra("all");
      }
     }

     $this->data->setValue("photoalbum_tagsmapping", $tags, $c);
    }
   }

   return true;
  }


  function getNameAndDate()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($directory=$this->data->getValue("directory", $c))) //directory set
    {
     if(preg_match("!((\d{4}-\d{2}-\d{2}) )?([^/]+)/$!", $directory, $m)) //date (optional) + name
     {
      if(!$this->data->getValue("name", $c)) //name not set
      {
       $this->data->setValue("name", $m[3], $c);

       if($m[2]) //set date (regardless whether it's set)
        $this->data->setValue("date", DT::dateConvertS($m[2], "Y-m-d", $this->data->field["date"]["format"][$this->data->dataformat]["dateformat"]), $c);
       else
        $this->data->unsetValue("date", $c);
      }
     }
     else //no detectable format
     {
      if(!$this->data->getValue("name", $c)) //name not set
       $this->data->setValue("name", $directory, $c); //use directory name
     }
    }
   }

   return true;
  }

  static function getCategory($type, $catnr=0)
  {
   $photoalbumcatnr=0;
   $PHOTOALBUMID="PHOTOALBUM CONTAINER";
   $COMMENTID="PHOTOALBUM COMMENTS CONTAINER";

   //get root category
   if($catnr) //sub category of container
   {
    //find container
    $d=Data::cache("categories", "tree", "up");
    $d->setShowFields("nr", "text");

    $d->setValue("catnr", $catnr);

    if(!$d->get())
     return false;

    while($d->next())
    {
     if($d->v["text"]==$PHOTOALBUMID)
     {
      $photoalbumcatnr=$d->v["nr"];
      break;
     }
    }

    if(!$photoalbumcatnr)
     return false;
   }
   else //no sub category
   {
    $d=Data::cache("categories", "tree", "down");
    $d->setShowFields("nr");

    $d->setValue("catnr", 0);
    $d->setValue("treedepth", 0);

    if(!$d->get()||!$d->next())
     return false;

    $rootcatnr=$d->v["nr"];

    //get photo album category
    $d->setAction("view");
    $d->setExtra("all");
    $d->clearValues();

    $d->setShowFields("nr", "module", "moduleLock");
    $d->setValue("catnr", $rootcatnr);
    $d->setValue("text", $PHOTOALBUMID);

    if(!$d->get()||!$d->next())
     return false;

    $photoalbumcatnr=$d->v["nr"];
   }

   if($type=="container"||$type=="tags")
    return $photoalbumcatnr;
   else if($type=="comments")
   {
    //get photo album comments category
    $d->setAction("view");
    $d->setExtra("all");
    $d->clearValues();

    $d->setShowFields("nr");
    $d->setValue("catnr", $photoalbumcatnr);
    $d->setValue("text", $COMMENTID);

    if(!$d->get()||!$d->next()) //no photo album comments category found
     return false;

    return $d->v["nr"];
   }
   else if($type=="albums")
   {
    //get photo album comments category
    $d->setAction("tree");
    $d->setExtra("down");
    $d->clearValues();

    $d->setShowFields("nr");
    $d->setValue("catnr", $photoalbumcatnr);

    if(!$d->get()) //no photo album categories found
     return false;

    if(!$d->vmax)
     return false;

    $catnrs=Array();
    while($d->next())
     $catnrs[]=$d->v["nr"];

    return $catnrs; //array
   }

   return false;
  }
 }
?>
