<?

 /***************************************************************************************
 * Filename: modules/pictures.php
 *
 * This file is part of the Photo album JAS plugin.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The Photo album JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/photoalbum/
 ***************************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["include"]."fs.php");

 require_once($_path["project"]["plugins"]."photoalbum/modules/categories.php");

 class Plugin_Module_photoalbum_pictures extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->data->title="Photo album pictures";

   $this->data->addField("file", "localfile", "all", false, 1, 65536, false, "File");

   $this->data->addField("subject", "text", "all", "title", 1, 255, false, "Subject");

   $this->data->addField("filename", "text", "show;sql", "", 0, 255, 0, "File name");
   $this->data->addField("filesize", "nr", "show;sql", "", 0, false, 0, "File size");

   //exif
   $this->data->addField("date", "datetime", "all", "", 0, false, false, "Date");
   $this->data->addField("width", "nr", "show;sql", "", 0, false, 0, "Width");
   $this->data->addField("height", "nr", "show;sql", "", 0, false, 0, "Height");

   $this->data->addField("color", "boolean", "show;sql", "", 0, false, true, "Color");
   $this->data->addField("aperture", "float", "show;sql", "", 0, false, false, "Aperture");
   $this->data->addField("apertureshow", "text", "show", "", 0, false, false, "Aperture");
   $this->data->addField("exposure", "float", "show;sql", "", 0, false, false, "Exposure");
   $this->data->addField("exposureshow", "text", "show", "", 0, false, false, "Exposure");
   $this->data->addField("iso", "nr", "show;sql", "", 0, false, false, "ISO speed");

   $this->data->addField("orientation", "enum", "show;sql", "", 0, 6, "top", "Picture orientation");
   $this->data->enumOptions("orientation", "top", "right", "bottom", "left");

   $this->data->addField("mirror", "boolean", "show;sql", "", false, false, false, "Picture mirrored");

   $this->data->addField("camera", "text", "show;sql", "", 0, 255, "Unknown", "Camera");
   $this->data->addField("camera_model", "text", "show;sql", "", 0, 255, "", "Camera model");
   $this->data->addField("camera_ccd", "text", "show;sql", "", 0, 25, false, "Camera CCD size");
   $this->data->addField("camera_lens", "text", "show;sql", "", 0, 50, false, "Camera lens");

   $this->data->addField("tags", "text", "show;form", "", 0, 255, false, "Tags");


   $this->data->defaultFields();

   //category
   if(!$this->data->check("properties", "reload", "catnr"))
    $this->data->field["catnr"]["properties"].=";reload";

   //comments
   $this->data->addReference("comments", "photoalbum_comments", "photoalbumpicturenr", "1:*", "nr");

   if(isset($this->data->field["comments"]))
    $this->data->field["comments"]["properties"].=";hidden";


   //tags
   $this->data->addReference("photoalbum_picturetagsmapping", "photoalbum_tags", "nr", "*:*", "nr");

   if(isset($this->data->field["photoalbum_picturetagsmapping"]))
    $this->data->field["photoalbum_picturetagsmapping"]["properties"].=";hidden";

   //unique picture per album
   $this->data->addUnique("uniqueInCategory", "catnr", "file");

   $this->data->sortOrder("date", "subject");
  }


  function postInit()
  {
   global $_path;

   $this->data->setFileDir("file", $_path["project"]["root"]."photoalbums");
   $this->data->setFileExtensions("file", "png", "PNG", "gif", "GIF", "jpg", "jpeg", "JPG");

   if(!$this->setAlbumDirectoryAndUnpublished())
    return false;

   return true;
  }


  /*
   * Pre get
   */

  function viewPreGetErrorCheck(){return $this->convertTextToTags();}

  function viewPreGet(){return $this->showFields();}
  function copyPreGet(){return $this->showFields();}
  function editPreGet(){return $this->showFields();}
  function movePreGet(){return $this->showFields();}
  function deletePreGet(){return $this->showFields();}
  function ownerPreGet(){return $this->showFields();}


  /*
   * Post get
   */

  function viewPostGet()
  {
   if(!$this->convertApertureExposureToText())
    return false;

   if(!$this->convertTagsToText())
    return false;

   if(!$this->setAlbumDirectoryAndUnpublished())
    return false;

   return true;
  }
  function copyPostGet(){return $this->viewPostGet();}
  function editPostGet(){return $this->viewPostGet();}
  function movePostGet(){return $this->viewPostGet();}
  function deletePostGet(){return $this->viewPostGet();}
  function ownerPostGet(){return $this->viewPostGet();}


  /*
   * Pre submit
   */

  function addPreSubmitErrorCheck()
  {
   if(!$this->updateExifInformation())
    return false;

   if(!$this->convertTextToTagsAndAdd())
    return false;

   if(!$this->setAlbumDirectoryAndUnpublished())
    return false;

   return true;
  }
  function editPreSubmitErrorCheck(){return $this->addPreSubmitErrorCheck();}


  /*
   * Support functions
   */

  function showFields()
  {
   if($this->data->check("use", "show", "exposureshow"))
    if(!$this->data->check("use", "show", "exposure"))
     $this->data->field["exposure"]["use"].=";show";

   if($this->data->check("use", "show", "apertureshow"))
    if(!$this->data->check("use", "show", "aperture"))
     $this->data->field["aperture"]["use"].=";show";

   return true;
  }


  function setAlbumDirectoryAndUnpublished()
  {
   global $_path, $_photoalbum_pictures_setdir;

   if($_photoalbum_pictures_setdir)
    return true;

   $_photoalbum_pictures_setdir=true;

   //set album directory
   $album=0;
   if($picture=$this->data->getValue("nr"))
   {
    $d=Data::cache($this->data->module, "view", "item");
    $d->reference["catnr"]["module"]="photoalbum_categories";
    unset($d->reference["catnr"]["data"]);

    $d->setValue("nr", $picture);

    $d->setShowFields("catnr");
    $d->getReferenceData("catnr")->setShowFields("directory");

    if($d->get(1)&&$d->next())
    {
     $album=$d->v["catnr"]["nr"];
     $this->data->setFileDir("file", $_path["project"]["root"]."photoalbums/".$d->v["catnr"]["directory"]);
    }
   }
   else if($album=$this->data->getValue("catnr"))
   {
    $d=Data::cache("photoalbum_categories", "view", "item");

    $d->setValue("nr", $album);

    $d->setShowFields("directory");

    if($d->get()&&$d->next())
     $this->data->setFileDir("file", $_path["project"]["root"]."photoalbums/".$d->v["directory"]);
    else
     $album=0;
   }

   //set unpublished pictures
   if($album)
   {
    $d=Data::cache($this->data->module, "view", "list");
    $d->setValue("catnr", $album);
    $d->setShowFields("file");

    if($d->get())
    {
     while($d->next())
     {
      if($d->v["nr"]==$picture) //skip this picture
       continue;

      if(($entry=array_search($d->v["file"], $this->data->listOptions["file"]))!==false)
      {
       unset($this->data->listOptions["file"][$entry]); //remove entry
       $this->data->listOptions["file"]=array_values($this->data->listOptions["file"]); //reindex
      }
     }
    }
   }

   $_photoalbum_pictures_setdir=false;

   return true;
  }


  function convertApertureExposureToText()
  {
   if($this->data->check("use", "show", "exposureshow"))
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if($tmp=$this->data->sqlValue("exposure", $c))
     {
      $tmp=number_format((1/$tmp), $this->data->field["exposure"]["format"][$this->data->dataformat]["decimals"], $this->data->field["exposure"]["format"][$this->data->dataformat]["decimalseparator"], $this->data->field["exposure"]["format"][$this->data->dataformat]["thousandsseparator"]);
      $tmp="1/".$tmp."s";

      $this->data->setValue("exposureshow", $tmp, $c);
     }
    }
   }

   if($this->data->check("use", "show", "apertureshow"))
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if($tmp=$this->data->sqlValue("aperture", $c))
     {
      $tmp=number_format($tmp, $this->data->field["aperture"]["format"][$this->data->dataformat]["decimals"], $this->data->field["aperture"]["format"][$this->data->dataformat]["decimalseparator"], $this->data->field["aperture"]["format"][$this->data->dataformat]["thousandsseparator"]);
      $tmp="f/".$tmp;

      $this->data->setValue("apertureshow", $tmp, $c);
     }
    }
   }

   return true;
  }

  function convertTagsToText()
  {
   if($this->data->check("use", "show", "tags"))
   {
    $d=&$this->data->reference["photoalbum_picturetagsmapping"]["data"]; //get photoalbum_tagsmapping

    //setup photoalbum_tags reference
    if(!isset($d->reference["photoalbum_tags"]["data"]))
     $d->initReferenceData("photoalbum_tags");

    $showFields=$d->reference["photoalbum_tags"]["data"]->check("use", "show"); //store show fields
    $d->reference["photoalbum_tags"]["data"]->setShowFields("tag");

    for($c=0;$c<$this->data->vmax;$c++)
    {
     $d->setAction("view");
     $d->setExtra("all");
     $d->clearValues();
     $d->setValue($this->data->module, $this->data->getValue("nr", $c));

     $tags=Array();
     if($d->get(1))
     {
      while($d->next())
       $tags[]=$d->v["photoalbum_tags"]["tag"];
     }

     $this->data->setValue("tags", implode(", ", $tags), $c);
    }

    $d->reference["photoalbum_tags"]["data"]->setShowFields($showFields); //restore show fields
    unset($d->reference["photoalbum_tags"]["data"]); //delete photoalbum_tags data object
   }

   return true;
  }

  function convertTextToTags()
  {
   $tagsmapping=0;
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("photoalbum_tags", "view", "all");
      $d->setShowFields("nr");
     }

     $x=0;
     $d->clearValues();
     foreach($text as $tag) //find correspondig tag nr
     {
      $d->setValue("tag", $tag, $x);
      $d->setOperant("tag", "OR", $x);
      $x++;
     }

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }
     else
     {
      if($d->vmax==0) //tag(s) not found
      {
       $this->data->setValue("photoalbum_picturetagsmapping", 0, $tagsmapping); //ensure no results are return (unless other tags are found)
       $tagsmapping++;
      }
      else
      {
       while($d->next())
       {
        if($tagsmapping>0)
         $this->data->setOperant("photoalbum_picturetagsmapping", "OR", $tagsmapping-1);

        $this->data->setValue("photoalbum_picturetagsmapping", $d->v["nr"], $tagsmapping);
        $tagsmapping++;
       }
      }
     }
    }
   }

   return true;
  }

  function convertTextToTagsAndAdd()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("photoalbum_tags", "view", "all");
      $d->setShowFields("nr");

      //get tags category
      if(!($tagsCatNR=photoalbum_categories::getCategory("tags")))
      {
       $this->data->e->add("Unable to add tags, no tags category found", "error");
       return false;
      }
     }

     foreach($text as $tag) //find correspondig tag nr
     {
      $d->clearValues();
      $d->setValue("tag", $tag);

      if($d->get()&&$d->vmax) //tag exists
       $tags[]=$d->getValue("nr"); //store tag+id
      else
      {
       //add tag to database
       $d->setAction("add");
       $d->setExtra("item");

       $d->clearValues();
       $d->setValue("tag", $tag);
       $d->setValue("catnr", $tagsCatNR);

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }

       $tags[]=$d->getValue("nr"); //store tag+id

       //reset data object
       $d->setAction("view");
       $d->setExtra("all");
      }
     }

     $this->data->setValue("photoalbum_picturetagsmapping", $tags, $c);
    }
   }

   return true;
  }


  function updateExifInformation()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($file=$this->data->getValue("file", $c))
    {
     $file=$this->data->field["file"]["localpath"]["directory"].$file;
     $filename=substr($file, strrpos($file, "/")+1);
     if(FS::existsS($file))
     {
      //set filename as default subject
      if(!$this->data->getValue("subject", $c))
       $this->data->setValue("subject", $filename, $c);

      //set file properties
      $this->data->setValue("filename", $filename, $c);
      $this->data->setValue("filesize", FS::sizeS($file), $c);

      //exif
      if(!($exif=@exif_read_data($file))) //no exif
      {
       $this->data->e->add("Unable to read Exif information for: \"".$file."\"", "warning");

       //set default values in add mode
       if($this->data->action=="add")
       {
        if(!$this->data->getValue("date", $c))
         $this->data->setValue("date", date($this->data->field["date"]["format"][$this->data->dataformat]["datetimeformat"], FS::getTimeS($file, "created")), $c);

        //get image info
        $imageinfo=getimagesize($file);

        //width
        if(isset($imageinfo[0]))
         $this->data->setValue("width", $imageinfo[0], $c);
        else
         $this->data->setDefaultValue("width", $c);

        //height
        if(isset($imageinfo[1]))
         $this->data->setValue("height", $imageinfo[1], $c);
        else
         $this->data->setDefaultValue("height", $c);
       }
      }
      else //exif available
      {
       //picture date
       if(!$this->data->getValue("date", $c))
       {
        if(isset($exif["DateTime"]))
         $this->data->setValue("date", DT::dateConvertS($exif["DateTime"], "Y:m:d H:i:s", $this->data->field["date"]["format"][$this->data->dataformat]["datetimeformat"]), $c);
        else if(isset($exif["DateTimeOriginal"]))
         $this->data->setValue("date", DT::dateConvertS($exif["DateTimeOriginal"], "Y:m:d H:i:s", $this->data->field["date"]["format"][$this->data->dataformat]["datetimeformat"]), $c);
        else if($this->data->action=="add") //set file created add mode
         $this->data->setValue("date", date($this->data->field["date"]["format"][$this->data->dataformat]["datetimeformat"], FS::getTimeS($file, "created")), $c);
       }

       //width
       if(isset($exif["COMPUTED"]["Width"]))
        $this->data->setValue("width", $exif["COMPUTED"]["Width"], $c);
       else if(isset($exif["ExifImageWidth"]))
        $this->data->setValue("width", $exif["ExifImageWidth"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("width", $c);

       //height
       if(isset($exif["COMPUTED"]["Height"]))
        $this->data->setValue("height", $exif["COMPUTED"]["Height"], $c);
       else if(isset($exif["ExifImageLength"]))
        $this->data->setValue("height", $exif["ExifImageLength"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("height", $c);

       //color
       if(isset($exif["COMPUTED"]["IsColor"]))
        $this->data->setValue("color", ($exif["COMPUTED"]["IsColor"]?true:false), $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("color", $c);

       //aperture
       if(isset($exif["COMPUTED"]["ApertureFNumber"]))
       {
        $tmp=preg_replace("/^.*?([0-9\.]+).*?$/", "\\1", $exif["COMPUTED"]["ApertureFNumber"]);
        $this->data->setValue("aperture", $tmp, $c);
       }
       else if(isset($exif["FNumber"]))
       {
        $tmp=explode("/", $exif["FNumber"]);
        $this->data->setValue("aperture", $tmp[0]/$tmp[1], $c);
       }
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("aperture", $c);

       //exposure
       if(isset($exif["ExposureTime"]))
       {
        $tmp=explode("/", $exif["ExposureTime"]);
        $this->data->setValue("exposure", $tmp[0]/$tmp[1], $c);
       }
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("exposure", $c);

       //iso
       if(isset($exif["ISOSpeedRatings"]))
        $this->data->setValue("iso", $exif["ISOSpeedRatings"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("iso", $c);

       //orientation / mirror
       if(isset($exif["Orientation"]))
       {
        if($exif["Orientation"]==1||$exif["Orientation"]==2)
         $this->data->setValue("orientation", "top", $c);
        else if($exif["Orientation"]==6||$exif["Orientation"]==7)
         $this->data->setValue("orientation", "left", $c);
        else if($exif["Orientation"]==3||$exif["Orientation"]==4)
         $this->data->setValue("orientation", "bottom", $c);
        else if($exif["Orientation"]==8||$exif["Orientation"]==5)
         $this->data->setValue("orientation", "right", $c);

        if($exif["Orientation"]==1||$exif["Orientation"]==3||$exif["Orientation"]==6||$exif["Orientation"]==8)
         $this->data->setValue("mirror", false, $c);
        else
         $this->data->setValue("mirror", true, $c);
       }
       else if($this->data->action=="add") //set default in add mode
       {
        $this->data->setDefaultValue("orientation", $c);
        $this->data->setDefaultValue("mirror", $c);
       }

       //camera
       if(isset($exif["Make"]))
        $this->data->setValue("camera", $exif["Make"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("camera", $c);

       //camera model
       if(isset($exif["Model"]))
       {
        $tmp=$exif["Model"];
        $tmp=str_replace($this->data->getValue("camera", $c)." ", "", $tmp); //remove camera make from model
        $this->data->setValue("camera_model", $tmp, $c);
       }
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("camera_model", $c);

       //camera ccd
       if(isset($exif["COMPUTED"]["CCDWidth"]))
        $this->data->setValue("camera_ccd", $exif["COMPUTED"]["CCDWidth"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("camera_ccd", $c);

       //camera lens
       if(isset($exif["UndefinedTag:0xA434"]))
        $this->data->setValue("camera_lens", $exif["UndefinedTag:0xA434"], $c);
       else if(isset($exif["UndefinedTag:0x0095"]))
        $this->data->setValue("camera_lens", $exif["UndefinedTag:0x0095"], $c);
       else if($this->data->action=="add") //set default in add mode
        $this->data->setDefaultValue("camera_ccd", $c);
      }
     }
    }

   }
   return true;
  }

 }
?>
